from __future__ import annotations

import asyncio
import logging
from dataclasses import KW_ONLY, dataclass
from typing import Any, Iterator

from pyrogram import Client, enums, errors, raw, types


@dataclass
class Cleaner:
    api_id: int | str
    api_hash: str
    _: KW_ONLY
    confirm_all: bool = False
    log_level: int | str = logging.DEBUG

    def __post_init__(self) -> None:
        self.setup_logger()
        self.client = Client(
            __package__,
            api_id=self.api_id,
            api_hash=self.api_hash,
            # app_version=self.app_version,
            # device_model=self.device_model,
            # system_version=self.system_version,
        )

    def setup_logger(self) -> Cleaner:
        self.log = logging.getLogger(__name__)
        self.log.setLevel(self.log_level)
        return self

    @staticmethod
    def iter_chunks(data: list, chunksize: int = 100) -> Iterator[list]:
        for i in range(0, len(data), chunksize):
            yield data[i : i + chunksize]

    @staticmethod
    def confirm(s: str) -> bool:
        return input(f"{s}? (y/N): ").lower().strip().startswith("y")

    async def delete_contacts(self) -> None:
        if not self.confirm_all and not self.confirm("Delete contacts"):
            self.log.info("Canceled")
            return
        try:
            contacts = await self.client.get_contacts()
            await self.client.delete_contacts([x.id for x in contacts])
            self.log.info("Contacts deleted successfully!")
        except Exception as e:
            self.log.exception(e)

    async def get_chats(self) -> list[types.Chat]:
        rv = []
        dialog: types.Dialog
        async for dialog in self.client.get_dialogs():
            rv.append(dialog.chat)
        return rv

    async def delete_private_chats(self) -> None:
        if not self.confirm_all and not self.confirm("Delete private chats"):
            self.log.warn("Canceled")
            return
        try:
            chats = await self.get_chats()
            for chat in chats:
                if chat.type not in [
                    enums.ChatType.PRIVATE,
                    enums.ChatType.BOT,
                ]:
                    continue
                self.log.debug(
                    "delete private chat: %d (%s)",
                    chat.id,
                    chat.first_name,
                )
                # TODO: нужно ли на всякий случай удалить сообщения?
                # message_ids = []
                # async for message in self.client.get_chat_history(
                #     chat_id=chat.id
                # ):
                #     message_ids.append(message.id)
                # for chunk in self.iter_chunks(message_ids):
                #     await self.client.delete_messages(
                #         chat_id=chat.id, message_ids=chunk, revoke=True
                #     )
                # https://stackoverflow.com/a/72766038
                peer = await self.client.resolve_peer(chat.id)
                self.log.debug(peer)
                await self.client.invoke(
                    raw.functions.messages.DeleteHistory(
                        peer=peer, max_id=0, revoke=True
                    )
                )
            self.log.info("private chats deleted successfully!")
        except Exception as e:
            self.log.exception(e)

    async def get_group_chats(self) -> list[types.Chat]:
        return [
            chat
            for chat in await self.get_chats()
            if chat.type
            in [
                enums.ChatType.GROUP,
                enums.ChatType.SUPERGROUP,
                enums.ChatType.CHANNEL,
            ]
        ]

    async def delete_own_messages(self, chat_id: int | str) -> None:
        self.log.debug("Search own messages group#%s", chat_id)
        message_ids = []
        message: types.Message
        async for message in self.client.search_messages(
            chat_id=chat_id,
            from_user="me",
        ):
            message_ids.append(message.id)
        if message_ids:
            for chunk in self.iter_chunks(message_ids):
                self.log.debug(
                    f"Delete message sequence: {', '.join(map(str, chunk))}"
                )
                await self.client.delete_messages(
                    chat_id=chat_id, message_ids=chunk, revoke=True
                )
            self.log.info(
                "Total deleted messages %s: %d", chat_id, len(message_ids)
            )

    async def delete_group_messages(self) -> None:
        if not self.confirm_all and not self.confirm("Delete group messages"):
            self.log.warn("Canceled")
            return
        try:
            chats = await self.get_group_chats()
            seen = set()
            while chats:
                chat = chats.pop()
                self.log.debug("%s - %s", chat.id, chat.title)
                # Избегаем повторное удаление сообщений в группах с комментариями
                if chat.id in seen:
                    self.log.debug("already seen: %s", chat.id)
                    continue
                seen.add(chat.id)
                try:
                    # Каналы имеют группы с комментариями к постам. В них вступать необязательно, а значит в списке чатов они не видны
                    if chat.type == enums.ChatType.CHANNEL:
                        # linked_chat (Chat (https://docs.pyrogram.org/api/types/Chat#pyrogram.types.Chat), optional) – The linked discussion group (in case of channels) or the linked channel (in case of supergroups). Returned only in get_chat() (https://docs.pyrogram.org/api/methods/get_chat.html#pyrogram.Client.get_chat).only in get_chat() (https://docs.pyrogram.org/api/methods/get_chat.html#pyrogram.Client.get_chat).
                        chat_info = await self.client.get_chat(chat_id=chat.id)
                        if linked_chat := getattr(chat_info, "linked_chat"):
                            # self.log.debug(linked_chat)
                            chats.append(linked_chat)
                        # Flood control
                        await asyncio.sleep(2.0)
                    # {
                    #     "_": "Chat",
                    #     "id": -1001XXXXXXXXX,
                    #     ...
                    #     "permissions": {
                    #         "_": "ChatPermissions",
                    #         "can_send_messages": true,
                    #         "can_send_media_messages": false,
                    #         "can_send_other_messages": false,
                    #         "can_send_polls": true,
                    #         "can_add_web_page_previews": false,
                    #         "can_change_info": false,
                    #         "can_invite_users": false,
                    #         "can_pin_messages": false
                    #     }
                    #     ...
                    # }
                    # if chat.permissions and chat.permissions.can_send_messages:
                    await self.delete_own_messages(chat.id)
                    # else:
                    #     self.log.debug(
                    #         "you dont have permission to send messages in grooup#%s",
                    #         chat.id,
                    #     )
                except errors.ChannelPrivate as e:
                    self.log.warn(e)
            self.log.info("Group messages deleted successfully!")
        except Exception as e:
            self.log.exception(e)

    async def leave_groups(self) -> bool:
        if not self.confirm_all and not self.confirm("Leave groups"):
            self.log.warn("Canceled")
            return
        try:
            chats = await self.get_group_chats()
            for chat in chats:
                self.log.debug(f"Leave group #{chat.id}")
                await self.client.leave_chat(chat.id)
            self.log.info("Groups leaved successfully!")
        except Exception as e:
            self.log.error(e)

    async def clean(self) -> None:
        await self.delete_contacts()
        await self.delete_group_messages()
        await self.delete_private_chats()
        await self.leave_groups()

    async def dump_chats(self) -> None:
        try:
            chats = await self.get_chats()
            print("[" + ",".join(map(str, chats)) + "]", flush=True)
        except Exception as e:
            self.log.exception(e)

    async def dump_me(self) -> None:
        try:
            print(await self.client.get_me())
        except Exception as e:
            self.log.exception(e)

    async def logout(self) -> None:
        try:
            print(await self.client.log_out())
        except Exception as e:
            self.log.exception(e)

    async def __aenter__(self) -> Cleaner:
        await self.client.start()
        return self

    async def __aexit__(self, *args: Any, **kwargs: Any) -> None:
        await self.client.stop()



import argparse
import logging
from os import getenv
from typing import Sequence

from .cleaner import Cleaner
from .utils import make_sync


class NameSpace(argparse.Namespace):
    yes_all: bool
    verbosity: int
    command: str


def parse_args(argv: Sequence[str] | None) -> NameSpace:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-y",
        "--yes",
        help="confirm all",
        action=argparse.BooleanOptionalAction,
    )
    parser.add_argument(
        "-v",
        "--verbosity",
        help="increase verbosity",
        action="count",
        default=0,
    )
    parser.set_defaults(command="clean")

    command_methods = {
        "delete_contacts": "delete contacts",
        "delete_group_messages": "delete any type messages in groups including own posts",
        "leave_groups": "leave groups",
        "delete_private_chats": "delete private chat messages",
        "dump_chats": "dump chats debug info",
        "dump_me": "dump loggined user debug info",
        "logout": "terminate current session",
    }

    subparsers = parser.add_subparsers(help="commands")

    for name, description in command_methods.items():
        _parser = subparsers.add_parser(name, help=description)
        _parser.set_defaults(command=name)
    return parser.parse_args(argv)


@make_sync
async def cli(argv: Sequence[str] | None = None) -> None:
    logging.basicConfig()
    args = parse_args(argv)
    log_lvl = max(logging.DEBUG, logging.WARNING - args.verbosity * 10)
    async with Cleaner(
        api_id=int(getenv("TG_API_ID", 24439609)),
        api_hash=getenv("TG_API_HASH", "425c5e04e10edd2913e971b64a82186d"),
        confirm_all=args.yes,
        log_level=log_lvl,
    ) as cleaner:
        await getattr(cleaner, args.command)()



import asyncio
import functools
from typing import Any, Awaitable, Callable


def make_sync(func: Awaitable) -> Callable:
    @functools.wraps(func)
    def wrapper(*args: Any, **kwargs: Any) -> Any:
        return asyncio.run(func(*args, **kwargs))

    return wrapper


def truncate_string(s: str, limit: int = 75, ellipsis: str = "…") -> str:
    return s[:limit] + bool(s[limit:]) * ellipsis



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['telegram_cleaner']

package_data = \
{'': ['*']}

install_requires = \
['pyrogram>=2.0.99,<3.0.0', 'tgcrypto>=1.2.5,<2.0.0']

entry_points = \
{'console_scripts': ['tg-clean = telegram_cleaner.cli:cli']}

setup_kwargs = {
    'name': 'telegram-cleaner',
    'version': '0.1.1',
    'description': '',
    'long_description': '# Telegram Cleaner ♻️\n\nDelete all your messages of any type.\n\n**🇷🇺 RUSSIAN DISCLAIMER**: Telegram не является анонимным и никогда им не был. Сотрудникам спецслужб известны телефонные номера около 30 миллионов пользователей Telegram из России, а равно и паспортные данных их владельцев. До 2020 года по телефону можно было найти любого пользователя. Чекистские подстилки массово скупали симки, вбивали в контакты тысячи случайных номеров, а потом сохраняли в базе связку id пользователя и номера телефона. Так собиралась пользовательская база, например, «Глаза Бога». С учетом того, что Роскомнадзор запустил бота для поиска экстремистских комментариев в сети, в т.ч. в Telegram, я настоятельно рекомендую вам потереть свои старые комментарии в каналах и группах. Также помните, что Telegram сотрудничает с ФСБ и другими спецслужбами и занимается выдачей террористов. Если сотрудники телеги получат на вас запрос от гэбни, то никто из них не удосужится выяснить, настоящий вы террорист ИГИЛ или это обычный спам запросами на неугодных режиму, они просто передадут ваши ip-адрес и номер телефона.\n\n**⚠️ WARNING**: before using this utility, you can save all your data using the desktop application: `Settings ` > ` Advanced` > `Export Telegram data`.\n\nInstallation:\n\n```bash\n# via pip\n$ pip install -U telegram-cleaner\n\n# via pipx\n$ pipx install telegram-cleaner\n```\n\nUsage:\n\n```bash\n# see help\n$ tg-clean -h\n\n# first save your chats\n$ tg-clean dump_chats > mychats.json\n\n# later you can extract data from this file using jq\n$ jq -r \'.[] | "\\( .id ) \\( .username  ) " + if has("title") then .title else "\\( .first_name ) \\( .last_name  )" end\' mychats.json\n777000 null Telegram null\n-1001436354653 nwsru NEWS.ru | Новости\n...\n\n# delete messages in group chats, comments, posts\n$ tg-clean -vv delete_group_messages\n\n# delete private chats without confirmation\n$ tg-clean -y delete_private_chats\n\n# delete all your messages of any type\n$ tg-clean\n\n# You can use you own API_ID and API_HASH\n# Also you can use .env files with zsh dotenv plugin\n# Add this lines to ~/.bashrc or ~/.zshrc\nexport TG_API_ID=6\nexport TG_API_HASH=eb06d4abfb49dc3eeb1aeb98ae0f581e\n```\n',
    'author': 'Senior YAML Developer',
    'author_email': 'yamldeveloper@proton.me',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
